#!/bin/sh
. `dirname $0`/../func.sh

init_dev
[ $? -ne 0 ] && exit 1

echo " select resolution"
echo " ================="
echo " 1 = 1920 x 1200 Full"
echo " 2 = 1280 x 1024 SXGA, with ROI"
echo " 3 = 1024 x 768  XGA, with ROI"
echo " 4 = 800  x 600  SVGA, with Skip2, ROI and Binning (col/row average)"
echo " 5 = 480  x 300  with Skip4 and Binning (col summed / row average)"
echo " 6 = 1920 x 1080 FullHD, with ROI"
echo " 7 = 1280 x 720  HD, with ROI"
read RESOLUTION
echo " Your select = $RESOLUTION"
case $RESOLUTION in
  "1") SENSOR_RES="1920x1200"; GRAB_RES="1920x1200"; FRAME_SIZE="width=1920,height=1200"; OFFSET_SENSOR="(8,8)";;
  "2") SENSOR_RES="1280x1024"; GRAB_RES="1280x1024"; FRAME_SIZE="width=1280,height=1024"; OFFSET_SENSOR="(328,88)";;
  "3") SENSOR_RES="1024x768"; GRAB_RES="1024x768"; FRAME_SIZE="width=1024,height=768"; OFFSET_SENSOR="(456,224)";;
  "4") SENSOR_RES="1600x1200"; GRAB_RES="800x600"; FRAME_SIZE="width=800,height=600"; OFFSET_SENSOR="(168,8)"; CTLS=( -c row_binning=1 -c col_binning=1 );;
  "5") SENSOR_RES="1920x1200"; GRAB_RES="480x300"; FRAME_SIZE="width=480,height=300"; OFFSET_SENSOR="(8,8)"; CTLS=( -c row_binning=1 -c col_binning=2 );;
  "6") SENSOR_RES="1920x1080"; GRAB_RES="1920x1080"; FRAME_SIZE="width=1920,height=1080"; OFFSET_SENSOR="(8,68)";;
  "7") SENSOR_RES="1280x720"; GRAB_RES="1280x720"; FRAME_SIZE="width=1280,height=720"; OFFSET_SENSOR="(328,248)";;
  *) SENSOR_RES="1920x1200"; GRAB_RES="1920x1200"; FRAME_SIZE="width=1920,height=1080"; OFFSET_SENSOR="(8,8)";;
esac

echo " starting gstreamer with $CAM_COL_FMT Source ..."
echo " read $FRAME_SIZE (offset x,y=$OFFSET_SENSOR) convert bayer2rgb and write to framebuffer"
echo " ==============================================================================="
echo ""
echo " configure CSI1 (CAM1 port) with media_control"
echo " ================================================"

media-ctl -r
if [ "$PHYCAML" = "0" ]; then
	media-ctl -l "'${CAM_ENTITY_NAME}':0->'imx7-mipi-csis.0':0[1]"
#           Camera -> imx7_mipi_csis.0 -> CSI -> CSI capture (/dev/videoX)

	media-ctl -V "'${CAM_ENTITY_NAME}':0 [fmt:${CAM_COL_FMT}/${GRAB_RES} ${OFFSET_SENSOR}/${SENSOR_RES}]"
	media-ctl -V "'imx7-mipi-csis.0':0 [fmt:${CAM_COL_FMT}/${GRAB_RES}]"
	media-ctl -V "'csi':0 [fmt:${CAM_COL_FMT}/${GRAB_RES} field:none]"
else
	media-ctl -l "'ds90ub953 4-0018':1->'ds90ub954 3-003d':0[1]"
	media-ctl -l "'ds90ub954 3-003d':2->'imx7-mipi-csis.0':0[1]"
#           Camera -> DS90ub953 -> DS90ub954 ->imx7_mipi_csis.0 -> CSI -> CSI capture (/dev/videoX)

	media-ctl -V "'${CAM_ENTITY_NAME}':0 [fmt:${CAM_COL_FMT}/${GRAB_RES} ${OFFSET_SENSOR}/${SENSOR_RES}]"
	media-ctl -V "'imx7-mipi-csis.0':0 [fmt:${CAM_COL_FMT}/${GRAB_RES}]"
	media-ctl -V "'csi':0 [fmt:${CAM_COL_FMT}/${GRAB_RES} field:none]"
fi


echo ""
echo " configure camera @CAM1 with v4l2_control"
echo " ========================================"

$V4L2_CTRL_CAM1_COL
v4l2-ctl "${CTLS[@]}" -d "$CAM_DEVICE"
#for higher fps reduce the exposure time, for example the auto_exposure_max in AEC Mode:
#v4l2-ctl -d $CAM_DEVICE -c auto_exposure_max=480

echo ""
echo " start gstreamer, break with ctl-C"
echo " ================================="

gst-launch-1.0 \
	v4l2src device=$VID_DEVICE ! \
	video/x-$COL_FORMAT,$FRAME_SIZE ! \
	bayer2rgbneon ! \
	waylandsink sync=false
#	fbdevsink sync=false																	# disable Wayland/Weston
#	kmssink driver-name="imx-drm" force-modesetting=false can-scale=false sync=false		# disable Wayland/Weston
#	waylandsink sync=false																	# enable Wayland/Weston

CTLS=( -c row_binning=0 -c col_binning=0)
v4l2-ctl "${CTLS[@]}" -d "$CAM_DEVICE"