#!/usr/bin/env python

'''
face detection using haar cascades

USAGE:
    facedetect.py [--cascade <cascade_fn>] [--nested-cascade <cascade_fn>] [<video_source>]
'''

# Python 2/3 compatibility
from __future__ import print_function

import numpy as np
import cv2
import subprocess
import os
import time


def detect(img, cascade):
    rects = cascade.detectMultiScale(img, scaleFactor=4, minNeighbors=3, minSize=(25, 25),
                                     flags=cv2.CASCADE_SCALE_IMAGE)
    if len(rects) == 0:
        return []
    rects[:,2:] += rects[:,:2]
    return rects

def draw_rects(img, rects, color):
    for x1, y1, x2, y2 in rects:
        cv2.rectangle(img, (x1, y1), (x2, y2), color, 2)

def main():
    import sys, getopt
    
    prev_frame_time = 0
    new_frame_time = 0
    fps_average_count = 0
    fps = 0
    font = cv2.FONT_HERSHEY_SIMPLEX
    TopLeftCornerOfText = (10,30)
    fontScale           = 1
    fontColor           = (255,255,255)
    lineType            = 2

    args, video_src = getopt.getopt(sys.argv[1:], '', ['cascade=', 'nested-cascade='])
    try:
        video_src = video_src[0]
    except:
        video_src = 0
    args = dict(args)
    cascade_fn = args.get('--cascade', "haarcascade_frontalface_default.xml")
#    cascade_fn = args.get('--cascade', "haarcascade_frontalface_alt.xml")
    nested_fn  = args.get('--nested-cascade', "haarcascade_eye.xml")

    cascade = cv2.CascadeClassifier(cv2.samples.findFile(cascade_fn))
    nested = cv2.CascadeClassifier(cv2.samples.findFile(nested_fn))

    print ("OpenCV version :  {0}".format(cv2.__version__))

    # read camera parameter at CSI1/CAM1 via ISI
    CSI1_DEVICE = ("/dev/video0" )
    print (" capture device for CSI1 port =", CSI1_DEVICE)
    CSI1_DEVICE_NUMBER = int(CSI1_DEVICE[10:].strip())	# .strip() -> cut the first 10 characters
    print (" capture device number for CSI1 port = ", CSI1_DEVICE_NUMBER)

    CAM1_DEVICE = ("/dev/v4l-subdev2" )
    print (" camera device for CSI1 port  =", CAM1_DEVICE)

    CAM_TYP = ("cat /sys/class/video4linux/video0/name")
    CAM_TYP = os.popen(str(CAM_TYP)).read()[:-1]				# cut the "\n" at the end
    print (" Camera Typ =", CAM_TYP)
    CAM_DRIVER = ( "echo %s | cut -d\" \" -f1" % CAM_TYP)
    CAM_DRIVER = os.popen(str(CAM_DRIVER)).read()[:-1]			# cut the "\n" at the end
    print (" Camera Driver Name =", CAM_DRIVER)

    COLOR_FMT = "v4l2-ctl -d /dev/cam-csi1 --get-subdev-fmt 0 | grep \"Mediabus Code\" | sed \'s/.*BUS_FMT_\\([A-Z]*\).*/\\1/g\'"
    COLOR_FMT = os.popen(str(COLOR_FMT)).read()[:-1]			# cut the "\n" at the end
    print (" Color Format =", COLOR_FMT)

    print ("")
    print ("choose you camera")
    print ("=================")
    print ("1 = VM-016-BW,  Resolution (1280 x 720) ROI")
    print ("2 = VM-016-COL, Resolution (1280 x 720) ROI")
    print ("3 = VM-017-BW,  Resolution (1280 x 720) ROI")
    print ("4 = VM-017-COL, Resolution (1280 x 720) ROI")
    print ("5 = VM-x20-BW,  Resolution (1280 x 720) ROI")
    print ("6 = VM-x20-COL, Resolution (1280 x 720) ROI")

    camera_version = input("Your select = ")

    # for other resolutions change the "cam_width", "cam_height" and "cam_offset" parameter
    if camera_version == "1":
        cam_width = 1280; cam_height = 720; cam_offset = "0,44"
        cam_typ = "'ar0144 3-0010'"; grab_res = str(cam_width)+"x"+str(cam_height); cam_col_fmt = "Y8"; grab_res_array = grab_res
        v4l2_ctl_param = "auto_exposure_max=720,dynamic_defect_pixel_correction=1"
    elif camera_version == "2":
        cam_width = 1280; cam_height = 720; cam_offset = "0,44"
        cam_typ = "'ar0144 3-0010'"; grab_res = str(cam_width)+"x"+str(cam_height); cam_col_fmt = "SGRBG8_1X8"; grab_res_array = grab_res
        v4l2_ctl_param = "auto_exposure_max=720,dynamic_defect_pixel_correction=0"
    elif camera_version == "3":
        cam_width = 1280; cam_height = 720; cam_offset = "16,252"
        cam_typ = "'ar0521 3-0036'"; grab_res = str(cam_width)+"x"+str(cam_height); grab_res_array = str(cam_width*2)+"x"+str(cam_height*2); cam_col_fmt = "Y8"
        v4l2_ctl_param = "exposure=2240,analogue_gain=4000,dynamic_defect_pixel_correction=1"
    elif camera_version == "4":
        cam_width = 1280; cam_height = 720; cam_offset = "16,252"
        cam_typ = "'ar0521 3-0036'"; grab_res = str(cam_width)+"x"+str(cam_height); grab_res_array = str(cam_width*2)+"x"+str(cam_height*2); cam_col_fmt = "SGRBG8_1X8"
        v4l2_ctl_param = "exposure=2240,analogue_gain=5000,digital_gain_red=1400,digital_gain_blue=1400,dynamic_defect_pixel_correction=1"
    elif camera_version == "5":
        cam_width = 1280; cam_height = 720; cam_offset = "320,240"
        cam_typ = "'ar0234 3-0010'"; grab_res = str(cam_width)+"x"+str(cam_height); cam_col_fmt = "Y8"; grab_res_array = grab_res
        v4l2_ctl_param = "auto_exposure=0,auto_exposure_max=1200,dynamic_defect_pixel_correction=1"
    elif camera_version == "6":
        cam_width = 1280; cam_height = 720; cam_offset = "320,240"
        cam_typ = "'ar0234 3-0010'"; grab_res = str(cam_width)+"x"+str(cam_height); cam_col_fmt = "SGRBG8_1X8"; grab_res_array = grab_res
        v4l2_ctl_param = "auto_exposure=0,auto_exposure_max=1200,digital_gain_red=1300,digital_gain_blue=1500,dynamic_defect_pixel_correction=0"
    else:
        print ("wrong number")
        sys.exit(0)

    print ("")
    print ("grab_res =",grab_res)
    print ("grab_res_array =",grab_res_array)

    # set display
    os.environ["DISPLAY"] = ':0.0'


    # read /dev/video[x] camera at CSI1/CAM1
    ret = subprocess.run(["media-ctl", "-e", "csi capture"], stdout=subprocess.PIPE)
    CSI1_DEVICE = ret.stdout.decode()					# .decode -> convert returnvalue from byte in string
    CSI1_DEVICE = CSI1_DEVICE[:-1]						# cut the "\n" at the end
    CSI1_DEVICE_NUMBER = int(CSI1_DEVICE[10:].strip())	# .strip() -> cut the first 10 characters
    print ("")
    print (" capture device for CSI1 port = ", CSI1_DEVICE)
    print (" capture device_number for CSI1 port = ", CSI1_DEVICE_NUMBER)

    PHYCAM_L = os.path.exists('/sys/bus/i2c/drivers/ds90ub953/4-0018')
    print (" phyCAM-L = ", PHYCAM_L)
    if PHYCAM_L == 1:
        cam_dummy = list(cam_typ)		# change i2c-bus number to 4
        cam_dummy[8] = '4'
        cam_typ = "".join(cam_dummy)
        print (" camtyp = ", cam_typ)

    cam_typ1 = cam_typ[:-1]								# cut last char '
    cam_typ1 = cam_typ1[1:]								# cut first char '
    ret = subprocess.run(['media-ctl', '-e', cam_typ1], stdout=subprocess.PIPE)
    CAM1_DEVICE = ret.stdout.decode()					# .decode -> convert returnvalue from byte in string
    CAM1_DEVICE = CAM1_DEVICE[:-1]						# cut the "\n" at the end
    print (" camera device for CSI1 port = ", CAM1_DEVICE)


    # set video path, camera resolution and ROI position via media-ctl
    print ("")
    print (" configure CSI1_DEVICE (camera_1 port) with media_ctl")
    print (" Note: For use other resolutions,")
    print (" see the scripts in path ...\\gstreamer_examples\\...")
    print (" ===============================================================")
    subprocess.run(["media-ctl", "-r"])
    if PHYCAM_L == 0:
        subprocess.run(["media-ctl", "-l", cam_typ+":0->'imx7-mipi-csis.0':0[1]"])

        subprocess.run(["media-ctl", "-V", cam_typ+":0 [fmt:"+cam_col_fmt+"/"+grab_res+" ("+cam_offset+")/"+grab_res_array+"]"])
        subprocess.run(["media-ctl", "-V", "'imx7-mipi-csis.0':0 [fmt:"+cam_col_fmt+"/"+grab_res+" field:none]"])
        subprocess.run(["media-ctl", "-V", "'csi':0 [fmt:"+cam_col_fmt+"/"+grab_res+" field:none]"])
    else:
        subprocess.run(["media-ctl", "-l", "'ds90ub953 4-0018':1->'ds90ub954 3-003d':0[1]"])
        subprocess.run(["media-ctl", "-l", "'ds90ub954 3-003d':2->'imx7-mipi-csis.0':0[1]"])

        subprocess.run(["media-ctl", "-V", cam_typ+":0 [fmt:"+cam_col_fmt+"/"+grab_res+" ("+cam_offset+")/"+grab_res_array+"]"])
        subprocess.run(["media-ctl", "-V", "'imx7-mipi-csis.0':0 [fmt:"+cam_col_fmt+"/"+grab_res+" field:none]"])
        subprocess.run(["media-ctl", "-V", "'csi':0 [fmt:"+cam_col_fmt+"/"+grab_res+" field:none]"])

    # set camera parameters via v4l2-ctl
    subprocess.run(["v4l2-ctl", "-d", str(CAM1_DEVICE), "-c", str(v4l2_ctl_param)])	# set v4l2 controls


    # open videodevice via GStreamer
    if cam_col_fmt == "SGRBG8_1X8":
        cap = cv2.VideoCapture("v4l2src device="+str(CSI1_DEVICE)+" ! video/x-bayer, format=grbg, width="+str(cam_width)+", height="+str(cam_height)+" ! appsink", cv2.CAP_GSTREAMER)
    else:
        cap = cv2.VideoCapture("v4l2src device="+str(CSI1_DEVICE)+" ! video/x-raw, format=GRAY8,depth=8, width="+str(cam_width)+", height="+str(cam_height)+" ! appsink", cv2.CAP_GSTREAMER)


    if not cap.isOpened():
        print(" Cannot open camera")
        exit()


#    cam = create_capture(video_src, fallback='synth:bg={}:noise=0.05'.format(cv2.samples.findFile('samples/data/lena.jpg')))

    while True:
        _ret, vis = cap.read()
        
        if cam_col_fmt == "SGRBG8_1X8":
            gray = cv2.cvtColor(vis, cv2.COLOR_BAYER_GB2GRAY)
            vis = cv2.cvtColor(vis, cv2.COLOR_BAYER_GB2BGR)
        else :
            gray = vis

        gray = cv2.equalizeHist(gray)

        rects = detect(gray, cascade)
        draw_rects(vis, rects, (0, 255, 0))
        if not nested.empty():
            for x1, y1, x2, y2 in rects:
                vis_roi = vis[y1:y2, x1:x2]
                subrects = detect(vis_roi.copy(), nested)
                draw_rects(vis_roi, subrects, (255, 0, 0))

        if fps_average_count == 5:
            new_frame_time = time.time()
            fps = 5/(new_frame_time-prev_frame_time)
            prev_frame_time = new_frame_time
            print ("fps = %3.1f"% fps, end='\r')
            fps_average_count = 0
        fps_average_count += 1
        cv2.putText(vis, "FPS: {:03.1f}".format(fps), TopLeftCornerOfText, font, fontScale, fontColor, lineType)

        cv2.imshow('facedetect', vis)

        if cv2.waitKey(5) == 27:
            break

    print('Done')


if __name__ == '__main__':
    print(__doc__)
    main()
    cv2.destroyAllWindows()
